@@ -1,3 +1,9 @@
+# ext/associationproxy.py
+# Copyright (C) 2005-2011 the SQLAlchemy authors and contributors <see AUTHORS file>
+#
+# This module is part of SQLAlchemy and is released under
+# the MIT License: http://www.opensource.org/licenses/mit-license.php
+
 """Contain the ``AssociationProxy`` class.
 
 The ``AssociationProxy`` is a Python property object which provides
@@ -13,6 +19,7 @@ from sqlalchemy import exceptions
 from sqlalchemy import orm
 from sqlalchemy import util
 from sqlalchemy.orm import collections
+from sqlalchemy.sql import not_
 
 
 def association_proxy(target_collection, attr, **kw):
@@ -29,13 +36,13 @@ def association_proxy(target_collection, attr, **kw):
     always in sync with *target_collection*, and mutations made to either
     collection will be reflected in both.
 
-    Implements a Python property representing a relation as a collection of
+    Implements a Python property representing a relationship as a collection of
     simpler values.  The proxied property will mimic the collection type of
-    the target (list, dict or set), or, in the case of a one to one relation,
+    the target (list, dict or set), or, in the case of a one to one relationship,
     a simple scalar value.
 
-    :param target_collection: Name of the relation attribute we'll proxy to,
-      usually created with :func:`~sqlalchemy.orm.relation`.
+    :param target_collection: Name of the relationship attribute we'll proxy to,
+      usually created with :func:`~sqlalchemy.orm.relationship`.
 
     :param attr: Attribute on the associated instances we'll proxy for.
 
@@ -43,7 +50,7 @@ def association_proxy(target_collection, attr, **kw):
       by this proxy property would look like [getattr(obj1, *attr*),
       getattr(obj2, *attr*)]
 
-      If the relation is one-to-one or otherwise uselist=False, then simply:
+      If the relationship is one-to-one or otherwise uselist=False, then simply:
       getattr(obj, *attr*)
 
     :param creator: optional.
@@ -57,14 +64,14 @@ def association_proxy(target_collection, attr, **kw):
       If you want to construct instances differently, supply a *creator*
       function that takes arguments as above and returns instances.
 
-      For scalar relations, creator() will be called if the target is None.
+      For scalar relationships, creator() will be called if the target is None.
       If the target is present, set operations are proxied to setattr() on the
       associated object.
 
       If you have an associated object with multiple attributes, you may set
       up multiple association proxies mapping to different attributes.  See
       the unit tests for examples, and for examples of how creator() functions
-      can be used to construct the scalar relation on-demand in this
+      can be used to construct the scalar relationship on-demand in this
       situation.
 
     :param \*\*kw: Passes along any other keyword arguments to
@@ -83,7 +90,7 @@ class AssociationProxy(object):
 
         target_collection
           Name of the collection we'll proxy to, usually created with
-          'relation()' in a mapper setup.
+          'relationship()' in a mapper setup.
 
         attr
           Attribute on the collected instances we'll proxy for.  For example,
@@ -116,7 +123,7 @@ class AssociationProxy(object):
           sniffing the target collection.  If your collection type can't be
           determined by duck typing or you'd like to use a different
           collection implementation, you may supply a factory function to
-          produce those collections.  Only applicable to non-scalar relations.
+          produce those collections.  Only applicable to non-scalar relationships.
 
         proxy_bulk_set
           Optional, use with proxy_factory.  See the _set() method for
@@ -172,7 +179,7 @@ class AssociationProxy(object):
             proxy = self._new(_lazy_collection(obj, self.target_collection))
             setattr(obj, self.key, (id(obj), proxy))
             return proxy
-    
+
     def __set__(self, obj, values):
         if self.owning_class is None:
             self.owning_class = type(obj)
@@ -220,13 +227,13 @@ class AssociationProxy(object):
         self.collection_class = util.duck_type_collection(lazy_collection())
 
         if self.proxy_factory:
-            return self.proxy_factory(lazy_collection, creator, self.value_attr)
+            return self.proxy_factory(lazy_collection, creator, self.value_attr, self)
 
         if self.getset_factory:
             getter, setter = self.getset_factory(self.collection_class, self)
         else:
             getter, setter = self._default_getset(self.collection_class)
-        
+
         if self.collection_class is list:
             return _AssociationList(lazy_collection, creator, getter, setter, self)
         elif self.collection_class is dict:
@@ -247,7 +254,7 @@ class AssociationProxy(object):
             getter, setter = self.getset_factory(self.collection_class, self)
         else:
             getter, setter = self._default_getset(self.collection_class)
-        
+
         proxy.creator = creator
         proxy.getter = getter
         proxy.setter = setter
@@ -266,6 +273,26 @@ class AssociationProxy(object):
                'no proxy_bulk_set supplied for custom '
                'collection_class implementation')
 
+    @property
+    def _comparator(self):
+        return self._get_property().comparator
+
+    def any(self, criterion=None, **kwargs):
+        return self._comparator.any(getattr(self.target_class, self.value_attr).has(criterion, **kwargs))
+
+    def has(self, criterion=None, **kwargs):
+        return self._comparator.has(getattr(self.target_class, self.value_attr).has(criterion, **kwargs))
+
+    def contains(self, obj):
+        return self._comparator.any(**{self.value_attr: obj})
+
+    def __eq__(self, obj):
+        return self._comparator.has(**{self.value_attr: obj})
+
+    def __ne__(self, obj):
+        return not_(self.__eq__(obj))
+
+
 class _lazy_collection(object):
     def __init__(self, obj, target):
         self.ref = weakref.ref(obj)
@@ -281,21 +308,21 @@ class _lazy_collection(object):
 
     def __getstate__(self):
         return {'obj':self.ref(), 'target':self.target}
-    
+
     def __setstate__(self, state):
         self.ref = weakref.ref(state['obj'])
         self.target = state['target']
 
 class _AssociationCollection(object):
     def __init__(self, lazy_collection, creator, getter, setter, parent):
-        """Constructs an _AssociationCollection.  
-        
+        """Constructs an _AssociationCollection.
+
         This will always be a subclass of either _AssociationList,
         _AssociationSet, or _AssociationDict.
 
         lazy_collection
           A callable returning a list-based collection of entities (usually an
-          object attribute managed by a SQLAlchemy relation())
+          object attribute managed by a SQLAlchemy relationship())
 
         creator
           A function that creates new target entities.  Given one parameter:
@@ -333,7 +360,7 @@ class _AssociationCollection(object):
         self.parent = state['parent']
         self.lazy_collection = state['lazy_collection']
         self.parent._inflate(self)
-    
+
 class _AssociationList(_AssociationCollection):
     """Generic, converting, list-to-list proxy."""
 
