@@ -9,11 +9,12 @@ from sqlalchemy import types, exc, schema
 from sqlalchemy.orm import *
 from sqlalchemy.sql import table, column
 from sqlalchemy.databases import mssql
-from sqlalchemy.dialects.mssql import pyodbc, mxodbc
+from sqlalchemy.dialects.mssql import pyodbc, mxodbc, pymssql
 from sqlalchemy.engine import url
 from sqlalchemy.test import *
 from sqlalchemy.test.testing import eq_, emits_warning_on, \
     assert_raises_message
+from sqlalchemy.engine.reflection import Inspector
 
 class CompileTest(TestBase, AssertsCompiledSQL):
     __dialect__ = mssql.dialect()
@@ -30,15 +31,15 @@ class CompileTest(TestBase, AssertsCompiledSQL):
                             'UPDATE sometable SET somecolumn=:somecolum'
                             'n WHERE sometable.somecolumn = '
                             ':somecolumn_1', dict(somecolumn=10))
-    
+
     # TODO: should this be for *all* MS-SQL dialects ?
     def test_mxodbc_binds(self):
         """mxodbc uses MS-SQL native binds, which aren't allowed in
         various places."""
-        
+
         mxodbc_dialect = mxodbc.dialect()
         t = table('sometable', column('foo'))
-        
+
         for expr, compile in [
             (
                 select([literal("x"), literal("y")]), 
@@ -60,7 +61,7 @@ class CompileTest(TestBase, AssertsCompiledSQL):
             )
         ]:
             self.assert_compile(expr, compile, dialect=mxodbc_dialect)
-    
+
     def test_in_with_subqueries(self):
         """Test that when using subqueries in a binary expression
         the == and != are changed to IN and NOT IN respectively.
@@ -127,7 +128,6 @@ class CompileTest(TestBase, AssertsCompiledSQL):
 
         s = table4.select(use_labels=True)
         c = s.compile(dialect=self.__dialect__)
-        print c.result_map
         assert table4.c.rem_id \
             in set(c.result_map['remote_owner_remotetable_rem_id'][1])
         self.assert_compile(table4.select(),
@@ -152,6 +152,19 @@ class CompileTest(TestBase, AssertsCompiledSQL):
                             'remote_owner.remotetable AS remotetable_1 '
                             'ON remotetable_1.rem_id = mytable.myid')
 
+        self.assert_compile(select([table4.c.rem_id,
+                table4.c.value]).apply_labels().union(select([table1.c.myid,
+                table1.c.description]).apply_labels()).alias().select(),
+                "SELECT anon_1.remote_owner_remotetable_rem_id, "
+                "anon_1.remote_owner_remotetable_value FROM "
+                "(SELECT remotetable_1.rem_id AS remote_owner_remotetable_rem_id, "
+                "remotetable_1.value AS remote_owner_remotetable_value "
+                "FROM remote_owner.remotetable AS remotetable_1 UNION "
+                "SELECT mytable.myid AS mytable_myid, mytable.description "
+                "AS mytable_description FROM mytable) AS anon_1"
+            )
+
+
     def test_delete_schema(self):
         metadata = MetaData()
         tbl = Table('test', metadata, Column('id', Integer,
@@ -427,24 +440,79 @@ class ReflectionTest(TestBase, ComparesTables):
         finally:
             meta.drop_all()
 
+    @testing.provide_metadata
     def test_identity(self):
-        meta = MetaData(testing.db)
         table = Table(
-            'identity_test', meta,
+            'identity_test', metadata,
             Column('col1', Integer, Sequence('fred', 2, 3), primary_key=True)
         )
         table.create()
 
         meta2 = MetaData(testing.db)
-        try:
-            table2 = Table('identity_test', meta2, autoload=True)
-            sequence = isinstance(table2.c['col1'].default, schema.Sequence) \
-                                    and table2.c['col1'].default
-            assert sequence.start == 2
-            assert sequence.increment == 3
-        finally:
-            table.drop()
+        table2 = Table('identity_test', meta2, autoload=True)
+        sequence = isinstance(table2.c['col1'].default, schema.Sequence) \
+                                and table2.c['col1'].default
+        assert sequence.start == 2
+        assert sequence.increment == 3
 
+    @testing.emits_warning("Did not recognize")
+    @testing.provide_metadata
+    def test_skip_types(self):
+        testing.db.execute("""
+            create table foo (id integer primary key, data xml)
+        """)
+        t1 = Table('foo', metadata, autoload=True)
+        assert isinstance(t1.c.id.type, Integer)
+        assert isinstance(t1.c.data.type, types.NullType)
+
+    @testing.provide_metadata
+    def test_indexes_cols(self):
+
+        t1 = Table('t', metadata, Column('x', Integer), Column('y', Integer))
+        Index('foo', t1.c.x, t1.c.y)
+        metadata.create_all()
+
+        m2 = MetaData()
+        t2 = Table('t', m2, autoload=True, autoload_with=testing.db)
+
+        eq_(
+            set(list(t2.indexes)[0].columns),
+            set([t2.c['x'], t2.c.y])
+        )
+
+    @testing.provide_metadata
+    def test_indexes_cols_with_commas(self):
+
+        t1 = Table('t', metadata, 
+                        Column('x, col', Integer, key='x'), 
+                        Column('y', Integer)
+                    )
+        Index('foo', t1.c.x, t1.c.y)
+        metadata.create_all()
+
+        m2 = MetaData()
+        t2 = Table('t', m2, autoload=True, autoload_with=testing.db)
+
+        eq_(
+            set(list(t2.indexes)[0].columns),
+            set([t2.c['x, col'], t2.c.y])
+        )
+
+    @testing.provide_metadata
+    def test_indexes_cols_with_spaces(self):
+
+        t1 = Table('t', metadata, Column('x col', Integer, key='x'), 
+                                    Column('y', Integer))
+        Index('foo', t1.c.x, t1.c.y)
+        metadata.create_all()
+
+        m2 = MetaData()
+        t2 = Table('t', m2, autoload=True, autoload_with=testing.db)
+
+        eq_(
+            set(list(t2.indexes)[0].columns),
+            set([t2.c['x col'], t2.c.y])
+        )
 
 class QueryUnicodeTest(TestBase):
 
@@ -477,24 +545,24 @@ class QueryTest(TestBase):
     def test_fetchid_trigger(self):
         """
         Verify identity return value on inserting to a trigger table.
-        
+
         MSSQL's OUTPUT INSERTED clause does not work for the
         case of a table having an identity (autoincrement)
         primary key column, and which also has a trigger configured
         to fire upon each insert and subsequently perform an
         insert into a different table. 
-        
+
         SQLALchemy's MSSQL dialect by default will attempt to
         use an OUTPUT_INSERTED clause, which in this case will
         raise the following error:
-        
+
         ProgrammingError: (ProgrammingError) ('42000', 334, 
         "[Microsoft][SQL Server Native Client 10.0][SQL Server]The 
         target table 't1' of the DML statement cannot have any enabled
         triggers if the statement contains an OUTPUT clause without
         INTO clause.", 7748) 'INSERT INTO t1 (descr) OUTPUT inserted.id
         VALUES (?)' ('hello',)
-        
+
         This test verifies a workaround, which is to rely on the
         older SCOPE_IDENTITY() call, which still works for this scenario.
         To enable the workaround, the Table must be instantiated
@@ -649,10 +717,10 @@ class SchemaTest(TestBase):
         dialect = mssql.dialect()
         self.ddl_compiler = dialect.ddl_compiler(dialect,
                 schema.CreateTable(t))
-    
+
     def _column_spec(self):
         return self.ddl_compiler.get_column_specification(self.column)
-        
+
     def test_that_mssql_default_nullability_emits_null(self):
         eq_("test_column VARCHAR NULL", self._column_spec())
 
@@ -799,12 +867,10 @@ class MatchTest(TestBase, AssertsCompiledSQL):
 
 
 class ParseConnectTest(TestBase, AssertsCompiledSQL):
-    __only_on__ = 'mssql'
-
     @classmethod
     def setup_class(cls):
         global dialect
-        dialect = pyodbc.MSDialect_pyodbc()
+        dialect = pyodbc.dialect()
 
     def test_pyodbc_connect_dsn_trusted(self):
         u = url.make_url('mssql://mydsn')
@@ -891,6 +957,26 @@ class ParseConnectTest(TestBase, AssertsCompiledSQL):
         eq_([['DRIVER={SQL Server};Server=hostspec;Database=database;UI'
             'D=username;PWD=password'], {}], connection)
 
+    def test_pymssql_port_setting(self):
+        dialect = pymssql.dialect()
+
+        u = \
+            url.make_url('mssql+pymssql://scott:tiger@somehost/test')
+        connection = dialect.create_connect_args(u)
+        eq_(
+            [[], {'host': 'somehost', 'password': 'tiger', 
+                    'user': 'scott', 'database': 'test'}], connection
+        )
+
+        u = \
+            url.make_url('mssql+pymssql://scott:tiger@somehost:5000/test')
+        connection = dialect.create_connect_args(u)
+        eq_(
+            [[], {'host': 'somehost:5000', 'password': 'tiger', 
+                    'user': 'scott', 'database': 'test'}], connection
+        )
+
+    @testing.only_on(['mssql+pyodbc', 'mssql+pymssql'], "FreeTDS specific test")
     def test_bad_freetds_warning(self):
         engine = engines.testing_engine()
 
@@ -912,7 +998,7 @@ class TypesTest(TestBase, AssertsExecutionResults, ComparesTables):
 
     def teardown(self):
         metadata.drop_all()
-    
+
     @testing.fails_on_everything_except('mssql+pyodbc',
             'this is some pyodbc-specific feature')
     def test_decimal_notation(self):
@@ -1408,7 +1494,7 @@ class TypesTest(TestBase, AssertsExecutionResults, ComparesTables):
                 elif c.name.startswith('int_n'):
                     assert not c.autoincrement, name
                     assert tbl._autoincrement_column is not c, name
-            
+
             # mxodbc can't handle scope_identity() with DEFAULT VALUES
 
             if testing.db.driver == 'mxodbc':
@@ -1421,7 +1507,7 @@ class TypesTest(TestBase, AssertsExecutionResults, ComparesTables):
                      : False}),
                      engines.testing_engine(options={'implicit_returning'
                      : True})]
-                    
+
             for counter, engine in enumerate(eng):
                 engine.execute(tbl.insert())
                 if 'int_y' in tbl.c:
@@ -1436,9 +1522,9 @@ class TypesTest(TestBase, AssertsExecutionResults, ComparesTables):
 
 class BinaryTest(TestBase, AssertsExecutionResults):
     """Test the Binary and VarBinary types"""
-    
+
     __only_on__ = 'mssql'
-    
+
     @classmethod
     def setup_class(cls):
         global binary_table, MyPickleType
@@ -1456,7 +1542,7 @@ class BinaryTest(TestBase, AssertsExecutionResults):
                     value.stuff = 'this is the right stuff'
                 return value
 
-        binary_table = Table(  
+        binary_table = Table(
             'binary_table',
             MetaData(testing.db),
             Column('primary_id', Integer, Sequence('binary_id_seq',
@@ -1537,3 +1623,38 @@ class BinaryTest(TestBase, AssertsExecutionResults):
         stream = fp.read(len)
         fp.close()
         return stream
+
+
+class ReflectHugeViewTest(TestBase):
+    __only_on__ = 'mssql'
+
+    def setup(self):
+        self.col_num = 150
+
+        self.metadata = MetaData(testing.db)
+        t = Table('base_table', self.metadata,
+                *[
+                    Column("long_named_column_number_%d" % i, Integer)
+                    for i in xrange(self.col_num)
+                ]
+        )
+        self.view_str = view_str = \
+            "CREATE VIEW huge_named_view AS SELECT %s FROM base_table" % (
+            ",".join("long_named_column_number_%d" % i 
+                        for i in xrange(self.col_num))
+            )
+        assert len(view_str) > 4000
+
+        DDL(view_str).execute_at('after-create', t)
+        DDL("DROP VIEW huge_named_view").execute_at('before-drop', t)
+
+        self.metadata.create_all()
+
+    def teardown(self):
+        self.metadata.drop_all()
+
+    def test_inspect_view_definition(self):
+        inspector = Inspector.from_engine(testing.db)
+        view_def = inspector.get_view_definition("huge_named_view")
+        eq_(view_def, self.view_str)
+
