@@ -1,8 +1,11 @@
-from sqlalchemy.test.testing import assert_raises, assert_raises_message
+from sqlalchemy.test.testing import assert_raises
+from sqlalchemy.test.testing import assert_raises_message
+from sqlalchemy.test.testing import emits_warning
+
 import pickle
 from sqlalchemy import Integer, String, UniqueConstraint, \
     CheckConstraint, ForeignKey, MetaData, Sequence, \
-    ForeignKeyConstraint, ColumnDefault
+    ForeignKeyConstraint, ColumnDefault, Index
 from sqlalchemy.test.schema import Table, Column
 from sqlalchemy import schema, exc
 import sqlalchemy as tsa
@@ -29,7 +32,7 @@ class MetaDataTest(TestBase, ComparesTables):
         t2 = Table('t2', metadata, Column('x', Integer), schema='foo')
         t3 = Table('t2', MetaData(), Column('x', Integer))
         t4 = Table('t1', MetaData(), Column('x', Integer), schema='foo')
-        
+
         assert "t1" in metadata
         assert "foo.t2" in metadata
         assert "t2" not in metadata
@@ -38,7 +41,7 @@ class MetaDataTest(TestBase, ComparesTables):
         assert t2 in metadata
         assert t3 not in metadata
         assert t4 not in metadata
-        
+
     def test_uninitialized_column_copy(self):
         for col in [
             Column('foo', String(), nullable=False),
@@ -46,13 +49,15 @@ class MetaDataTest(TestBase, ComparesTables):
             Column(Integer(), primary_key=True),
             Column('bar', Integer(), Sequence('foo_seq'), primary_key=True,
                                                             key='bar'),
-            Column(Integer(), ForeignKey('bat.blah')),
+            Column(Integer(), ForeignKey('bat.blah'), doc="this is a col"),
             Column('bar', Integer(), ForeignKey('bat.blah'), primary_key=True,
                                                             key='bar'),
+            Column('bar', Integer(), info={'foo':'bar'}),
         ]:
             c2 = col.copy()
             for attr in ('name', 'type', 'nullable', 
-                        'primary_key', 'key', 'unique'):
+                        'primary_key', 'key', 'unique', 'info',
+                        'doc'):
                 eq_(getattr(col, attr), getattr(c2, attr))
             eq_(len(col.foreign_keys), len(c2.foreign_keys))
             if col.default:
@@ -72,8 +77,8 @@ class MetaDataTest(TestBase, ComparesTables):
             cx = c1.copy()
             t = Table('foo%d' % i, m, cx)
         eq_(msgs, ['attach foo0.foo', 'attach foo1.foo', 'attach foo2.foo'])
-        
-        
+
+
     def test_dupe_tables(self):
         metadata = MetaData()
         t1 = Table('table1', metadata, 
@@ -97,28 +102,42 @@ class MetaDataTest(TestBase, ComparesTables):
                     "Table object."
         finally:
             metadata.drop_all()
-    
+
     def test_fk_copy(self):
         c1 = Column('foo', Integer)
         c2 = Column('bar', Integer)
         m = MetaData()
         t1 = Table('t', m, c1, c2)
-        
+
         kw = dict(onupdate="X", 
                         ondelete="Y", use_alter=True, name='f1',
                         deferrable="Z", initially="Q", link_to_name=True)
-                        
+
         fk1 = ForeignKey(c1, **kw) 
         fk2 = ForeignKeyConstraint((c1,), (c2,), **kw)
-        
+
         t1.append_constraint(fk2)
         fk1c = fk1.copy()
         fk2c = fk2.copy()
-        
+
         for k in kw:
             eq_(getattr(fk1c, k), kw[k])
             eq_(getattr(fk2c, k), kw[k])
-    
+
+    def test_check_constraint_copy(self):
+        r = lambda x: x
+        c = CheckConstraint("foo bar", 
+                            name='name', 
+                            initially=True, 
+                            deferrable=True, 
+                            _create_rule = r)
+        c2 = c.copy()
+        eq_(c2.name, 'name')
+        eq_(str(c2.sqltext), "foo bar")
+        eq_(c2.initially, True)
+        eq_(c2.deferrable, True)
+        assert c2._create_rule is r
+
     def test_fk_construct(self):
         c1 = Column('foo', Integer)
         c2 = Column('bar', Integer)
@@ -126,7 +145,7 @@ class MetaDataTest(TestBase, ComparesTables):
         t1 = Table('t', m, c1, c2)
         fk1 = ForeignKeyConstraint(('foo', ), ('bar', ), table=t1)
         assert fk1 in t1.constraints
-        
+
     @testing.exclude('mysql', '<', (4, 1, 1), 'early types are squirrely')
     def test_to_metadata(self):
         meta = MetaData()
@@ -218,7 +237,7 @@ class MetaDataTest(TestBase, ComparesTables):
                     assert not c.columns.contains_column(table.c.name)
         finally:
             meta.drop_all(testing.db)
-    
+
     def test_tometadata_with_schema(self):
         meta = MetaData()
 
@@ -246,6 +265,62 @@ class MetaDataTest(TestBase, ComparesTables):
         eq_(str(table_c.join(table2_c).onclause),
             'someschema.mytable.myid = someschema.othertable.myid')
 
+    def test_tometadata_kwargs(self):
+        meta = MetaData()
+
+        table = Table('mytable', meta,
+            Column('myid', Integer, primary_key=True),
+            mysql_engine='InnoDB',
+        )
+
+        meta2 = MetaData()
+        table_c = table.tometadata(meta2)
+
+        eq_(table.kwargs,table_c.kwargs)
+
+    def test_tometadata_indexes(self):
+        meta = MetaData()
+
+        table = Table('mytable', meta,
+            Column('id', Integer, primary_key=True),
+            Column('data1', Integer, index=True),
+            Column('data2', Integer),
+        )
+        Index('multi',table.c.data1,table.c.data2),
+
+        meta2 = MetaData()
+        table_c = table.tometadata(meta2)
+
+        def _get_key(i):
+            return [i.name,i.unique] + \
+                    sorted(i.kwargs.items()) + \
+                    i.columns.keys()
+
+        eq_(
+            sorted([_get_key(i) for i in table.indexes]),
+            sorted([_get_key(i) for i in table_c.indexes])
+        )
+
+    @emits_warning("Table '.+' already exists within the given MetaData")
+    def test_tometadata_already_there(self):
+
+        meta1 = MetaData()
+        table1 = Table('mytable', meta1,
+            Column('myid', Integer, primary_key=True),
+        )
+        meta2 = MetaData()
+        table2 = Table('mytable', meta2,
+            Column('yourid', Integer, primary_key=True),
+        )
+
+        meta3 = MetaData()
+
+        table_c = table1.tometadata(meta2)
+        table_d = table2.tometadata(meta2)
+
+        # d'oh!
+        assert table_c is table_d
+
     def test_tometadata_default_schema(self):
         meta = MetaData()
 
@@ -282,7 +357,7 @@ class MetaDataTest(TestBase, ComparesTables):
         c = Table('c', meta, Column('foo', Integer))
         d = Table('d', meta, Column('foo', Integer))
         e = Table('e', meta, Column('foo', Integer))
-        
+
         e.add_is_dependent_on(c)
         a.add_is_dependent_on(b)
         b.add_is_dependent_on(d)
@@ -292,8 +367,8 @@ class MetaDataTest(TestBase, ComparesTables):
             meta.sorted_tables,
             [d, b, a, c, e]
         )
-        
-        
+
+
     def test_tometadata_strip_schema(self):
         meta = MetaData()
 
@@ -332,7 +407,7 @@ class TableOptionsTest(TestBase, AssertsCompiledSQL):
         table1 = Table("temporary_table_1", MetaData(),
                       Column("col1", Integer),
                       prefixes = ["TEMPORARY"])
-                      
+
         self.assert_compile(
             schema.CreateTable(table1), 
             "CREATE TEMPORARY TABLE temporary_table_1 (col1 INTEGER)"
