@@ -3,7 +3,8 @@ from sqlalchemy.test.testing import eq_, assert_raises, assert_raises_message
 from sqlalchemy import *
 from sqlalchemy import exc as sa_exc, util
 from sqlalchemy.orm import *
-from sqlalchemy.orm import exc as orm_exc
+from sqlalchemy.orm import exc as orm_exc, attributes
+from sqlalchemy.test.assertsql import AllOf, CompiledSQL
 
 from sqlalchemy.test import testing, engines
 from sqlalchemy.util import function_named
@@ -84,7 +85,7 @@ class PolymorphicOnNotLocalTest(_base.MappedTest):
                 Column('id', Integer, primary_key=True), 
                 Column('y', String(10)), 
                 Column('xid', ForeignKey('t1.id')))
-    
+
     @testing.resolve_artifact_names
     def test_bad_polymorphic_on(self):
         class InterfaceBase(object):
@@ -126,8 +127,8 @@ class PolymorphicOnNotLocalTest(_base.MappedTest):
             "polymorphic loads will not function properly",
             go
         )
-        
-        
+
+
 
 class FalseDiscriminatorTest(_base.MappedTest):
     @classmethod
@@ -136,7 +137,7 @@ class FalseDiscriminatorTest(_base.MappedTest):
         t1 = Table('t1', metadata, 
             Column('id', Integer, primary_key=True, test_needs_autoincrement=True), 
             Column('type', Boolean, nullable=False))
-        
+
     def test_false_on_sub(self):
         class Foo(object):pass
         class Bar(Foo):pass
@@ -162,7 +163,7 @@ class FalseDiscriminatorTest(_base.MappedTest):
         assert d1.type is False
         sess.expunge_all()
         assert sess.query(Ding).one() is not None
-        
+
 class PolymorphicSynonymTest(_base.MappedTest):
     @classmethod
     def define_tables(cls, metadata):
@@ -176,7 +177,7 @@ class PolymorphicSynonymTest(_base.MappedTest):
                    Column('id', Integer, ForeignKey('t1.id'),
                                             primary_key=True),
                    Column('data', String(10), nullable=False))
-    
+
     def test_polymorphic_synonym(self):
         class T1(_fixtures.Base):
             def info(self):
@@ -184,9 +185,9 @@ class PolymorphicSynonymTest(_base.MappedTest):
             def _set_info(self, x):
                 self._info = x
             info = property(info, _set_info)
-            
+
         class T2(T1):pass
-        
+
         mapper(T1, t1, polymorphic_on=t1.c.type, polymorphic_identity='t1',
             properties={
                 'info':synonym('_info', map_column=True)
@@ -201,8 +202,8 @@ class PolymorphicSynonymTest(_base.MappedTest):
         sess.expunge_all()
         eq_(sess.query(T2).filter(T2.info=='at2').one(), at2)
         eq_(at2.info, "THE INFO IS:at2")
-        
-    
+
+
 class CascadeTest(_base.MappedTest):
     """that cascades on polymorphic relationships continue
     cascading along the path of the instance's mapper, not
@@ -303,13 +304,13 @@ class M2OUseGetTest(_base.MappedTest):
             # the 'primaryjoin' looks just like "Sub"'s "get" clause (based on the Base id),
             # and foreign_keys since that join condition doesn't actually have any fks in it
             #'sub':relationship(Sub, primaryjoin=base.c.id==related.c.sub_id, foreign_keys=related.c.sub_id)
-            
+
             # now we can use this:
             'sub':relationship(Sub)
         })
-        
+
         assert class_mapper(Related).get_property('sub').strategy.use_get
-        
+
         sess = create_session()
         s1 = Sub()
         r1 = Related(sub=s1)
@@ -322,7 +323,7 @@ class M2OUseGetTest(_base.MappedTest):
         def go():
             assert r1.sub
         self.assert_sql_count(testing.db, go, 0)
-        
+
 
 class GetTest(_base.MappedTest):
     @classmethod
@@ -342,7 +343,7 @@ class GetTest(_base.MappedTest):
             Column('foo_id', Integer, ForeignKey('foo.id')),
             Column('bar_id', Integer, ForeignKey('bar.id')),
             Column('data', String(20)))
-    
+
     @classmethod
     def setup_classes(cls):
         class Foo(_base.BasicEntity):
@@ -356,7 +357,7 @@ class GetTest(_base.MappedTest):
 
     def test_get_polymorphic(self):
         self._do_get_test(True)
-    
+
     def test_get_nonpolymorphic(self):
         self._do_get_test(False)
 
@@ -394,7 +395,7 @@ class GetTest(_base.MappedTest):
                 assert sess.query(Blub).get(f.id) is None
                 assert sess.query(Blub).get(b.id) is None
                 assert sess.query(Bar).get(f.id) is None
-                
+
             self.assert_sql_count(testing.db, go, 0)
         else:
             # this is testing the 'wrong' behavior of using get()
@@ -423,7 +424,7 @@ class GetTest(_base.MappedTest):
 class EagerLazyTest(_base.MappedTest):
     """tests eager load/lazy load of child items off inheritance mappers, tests that
     LazyLoader constructs the right query condition."""
-    
+
     @classmethod
     def define_tables(cls, metadata):
         global foo, bar, bar_foo
@@ -469,7 +470,7 @@ class EagerLazyTest(_base.MappedTest):
 class EagerTargetingTest(_base.MappedTest):
     """test a scenario where joined table inheritance might be 
     confused as an eagerly loaded joined table."""
-    
+
     @classmethod
     def define_tables(cls, metadata):
         Table('a_table', metadata,
@@ -483,14 +484,14 @@ class EagerTargetingTest(_base.MappedTest):
            Column('id', Integer, ForeignKey('a_table.id'), primary_key=True),
            Column('b_data', String(50)),
         )
-    
+
     @testing.resolve_artifact_names
     def test_adapt_stringency(self):
         class A(_base.ComparableEntity):
             pass
         class B(A):
             pass
-        
+
         mapper(A, a_table, polymorphic_on=a_table.c.type, polymorphic_identity='A', 
                 properties={
                     'children': relationship(A, order_by=a_table.c.name)
@@ -499,7 +500,7 @@ class EagerTargetingTest(_base.MappedTest):
         mapper(B, b_table, inherits=A, polymorphic_identity='B', properties={
                 'b_derived':column_property(b_table.c.b_data + "DATA")
                 })
-        
+
         sess=create_session()
 
         b1=B(id=1, name='b1',b_data='i')
@@ -516,15 +517,15 @@ class EagerTargetingTest(_base.MappedTest):
         node = sess.query(B).filter(B.id==bid).all()[0]
         eq_(node, B(id=1, name='b1',b_data='i'))
         eq_(node.children[0], B(id=2, name='b2',b_data='l'))
-        
+
         sess.expunge_all()
         node = sess.query(B).options(joinedload(B.children)).filter(B.id==bid).all()[0]
         eq_(node, B(id=1, name='b1',b_data='i'))
         eq_(node.children[0], B(id=2, name='b2',b_data='l'))
-        
+
 class FlushTest(_base.MappedTest):
     """test dependency sorting among inheriting mappers"""
-    
+
     @classmethod
     def define_tables(cls, metadata):
         Table('users', metadata,
@@ -727,7 +728,7 @@ class VersioningTest(_base.MappedTest):
 class DistinctPKTest(_base.MappedTest):
     """test the construction of mapper.primary_key when an inheriting relationship
     joins on a column other than primary key column."""
-    
+
     run_inserts = 'once'
     run_deletes = None
 
@@ -772,7 +773,7 @@ class DistinctPKTest(_base.MappedTest):
         mapper(Employee, employee_table, inherits=person_mapper,
                         properties={'pid':person_table.c.id, 
                                     'eid':employee_table.c.id})
-        self._do_test(True)
+        self._do_test(False)
 
     def test_explicit_composite_pk(self):
         person_mapper = mapper(Person, person_table)
@@ -811,7 +812,7 @@ class DistinctPKTest(_base.MappedTest):
 
 class SyncCompileTest(_base.MappedTest):
     """test that syncrules compile properly on custom inherit conds"""
-    
+
     @classmethod
     def define_tables(cls, metadata):
         global _a_table, _b_table, _c_table
@@ -880,17 +881,17 @@ class SyncCompileTest(_base.MappedTest):
 
 class OverrideColKeyTest(_base.MappedTest):
     """test overriding of column attributes."""
-    
+
     @classmethod
     def define_tables(cls, metadata):
         global base, subtable
-        
+
         base = Table('base', metadata, 
             Column('base_id', Integer, primary_key=True, test_needs_autoincrement=True),
             Column('data', String(255)),
             Column('sqlite_fixer', String(10))
             )
-            
+
         subtable = Table('subtable', metadata,
             Column('base_id', Integer, ForeignKey('base.base_id'), primary_key=True),
             Column('subdata', String(255))
@@ -905,7 +906,7 @@ class OverrideColKeyTest(_base.MappedTest):
 
         mapper(Base, base)
         mapper(Sub, subtable, inherits=Base)
-        
+
         # Sub gets a "base_id" property using the "base_id"
         # column of both tables.
         eq_(
@@ -917,12 +918,12 @@ class OverrideColKeyTest(_base.MappedTest):
         # this pattern is what you see when using declarative
         # in particular, here we do a "manual" version of
         # what we'd like the mapper to do.
-        
+
         class Base(object):
             pass
         class Sub(Base):
             pass
-        
+
         mapper(Base, base, properties={
             'id':base.c.base_id
         })
@@ -943,7 +944,7 @@ class OverrideColKeyTest(_base.MappedTest):
         sess.add(s1)
         sess.flush()
         assert sess.query(Sub).get(10) is s1
-    
+
     def test_override_onlyinparent(self):
         class Base(object):
             pass
@@ -954,7 +955,7 @@ class OverrideColKeyTest(_base.MappedTest):
             'id':base.c.base_id
         })
         mapper(Sub, subtable, inherits=Base)
-        
+
         eq_(
             class_mapper(Sub).get_property('id').columns,
             [base.c.base_id]
@@ -964,30 +965,30 @@ class OverrideColKeyTest(_base.MappedTest):
             class_mapper(Sub).get_property('base_id').columns,
             [subtable.c.base_id]
         )
-        
+
         s1 = Sub()
         s1.id = 10
-        
+
         s2 = Sub()
         s2.base_id = 15
-        
+
         sess = create_session()
         sess.add_all([s1, s2])
         sess.flush()
-        
+
         # s1 gets '10'
         assert sess.query(Sub).get(10) is s1
-        
+
         # s2 gets a new id, base_id is overwritten by the ultimate
         # PK col
         assert s2.id == s2.base_id != 15
-    
+
     @testing.emits_warning(r'Implicit')
     def test_override_implicit(self):
         # this is how the pattern looks intuitively when 
         # using declarative.
         # fixed as part of [ticket:1111]
-        
+
         class Base(object):
             pass
         class Sub(Base):
@@ -999,17 +1000,17 @@ class OverrideColKeyTest(_base.MappedTest):
         mapper(Sub, subtable, inherits=Base, properties={
             'id':subtable.c.base_id
         })
-        
+
         # Sub mapper compilation needs to detect that "base.c.base_id"
         # is renamed in the inherited mapper as "id", even though
         # it has its own "id" property.  Sub's "id" property 
         # gets joined normally with the extra column.
-        
+
         eq_(
             set(class_mapper(Sub).get_property('id').columns),
             set([base.c.base_id, subtable.c.base_id])
         )
-        
+
         s1 = Sub()
         s1.id = 10
         sess = create_session()
@@ -1019,7 +1020,7 @@ class OverrideColKeyTest(_base.MappedTest):
 
     def test_plain_descriptor(self):
         """test that descriptors prevent inheritance from propigating properties to subclasses."""
-        
+
         class Base(object):
             pass
         class Sub(Base):
@@ -1029,7 +1030,7 @@ class OverrideColKeyTest(_base.MappedTest):
 
         mapper(Base, base)
         mapper(Sub, subtable, inherits=Base)
-        
+
         s1 = Sub()
         sess = create_session()
         sess.add(s1)
@@ -1044,7 +1045,7 @@ class OverrideColKeyTest(_base.MappedTest):
                 if instance is None:
                     return self
                 return "im the data"
-            
+
         class Base(object):
             pass
         class Sub(Base):
@@ -1058,7 +1059,7 @@ class OverrideColKeyTest(_base.MappedTest):
         sess.add(s1)
         sess.flush()
         assert sess.query(Sub).one().data == "im the data"
-    
+
     def test_sub_columns_over_base_descriptors(self):
         class Base(object):
             @property
@@ -1070,7 +1071,7 @@ class OverrideColKeyTest(_base.MappedTest):
 
         mapper(Base, base)
         mapper(Sub, subtable, inherits=Base)
-        
+
         sess = create_session()
         b1 = Base()
         assert b1.subdata == "this is base"
@@ -1081,7 +1082,7 @@ class OverrideColKeyTest(_base.MappedTest):
         sess.add_all([s1, b1])
         sess.flush()
         sess.expunge_all()
-        
+
         assert sess.query(Base).get(b1.base_id).subdata == "this is base"
         assert sess.query(Sub).get(s1.base_id).subdata == "this is sub"
 
@@ -1112,36 +1113,43 @@ class OverrideColKeyTest(_base.MappedTest):
 
 class OptimizedLoadTest(_base.MappedTest):
     """tests for the "optimized load" routine."""
-    
+
     @classmethod
     def define_tables(cls, metadata):
-        global base, sub, with_comp
-        base = Table('base', metadata,
+        Table('base', metadata,
             Column('id', Integer, primary_key=True, test_needs_autoincrement=True),
             Column('data', String(50)),
-            Column('type', String(50))
+            Column('type', String(50)),
+            Column('counter', Integer, server_default="1")
         )
-        sub = Table('sub', metadata, 
+        Table('sub', metadata, 
             Column('id', Integer, ForeignKey('base.id'), primary_key=True),
-            Column('sub', String(50))
+            Column('sub', String(50)),
+            Column('counter', Integer, server_default="1"),
+            Column('counter2', Integer, server_default="1")
         )
-        with_comp = Table('with_comp', metadata,
+        Table('subsub', metadata,
+            Column('id', Integer, ForeignKey('sub.id'), primary_key=True),
+            Column('counter2', Integer, server_default="1")
+        )
+        Table('with_comp', metadata,
             Column('id', Integer, ForeignKey('base.id'), primary_key=True),
             Column('a', String(10)),
             Column('b', String(10))
         )
-    
+
+    @testing.resolve_artifact_names
     def test_optimized_passes(self):
         """"test that the 'optimized load' routine doesn't crash when 
         a column in the join condition is not available."""
-        
+
         class Base(_base.BasicEntity):
             pass
         class Sub(Base):
             pass
-            
+
         mapper(Base, base, polymorphic_on=base.c.type, polymorphic_identity='base')
-        
+
         # redefine Sub's "id" to favor the "id" col in the subtable.
         # "id" is also part of the primary join condition
         mapper(Sub, sub, inherits=Base, 
@@ -1152,7 +1160,7 @@ class OptimizedLoadTest(_base.MappedTest):
         sess.add(s1)
         sess.commit()
         sess.expunge_all()
-        
+
         # load s1 via Base.  s1.id won't populate since it's relative to 
         # the "sub" table.  The optimized load kicks in and tries to 
         # generate on the primary join, but cannot since "id" is itself unloaded.
@@ -1160,6 +1168,7 @@ class OptimizedLoadTest(_base.MappedTest):
         s1 = sess.query(Base).first()
         assert s1.sub == 's1sub'
 
+    @testing.resolve_artifact_names
     def test_column_expression(self):
         class Base(_base.BasicEntity):
             pass
@@ -1177,6 +1186,7 @@ class OptimizedLoadTest(_base.MappedTest):
         s1 = sess.query(Base).first()
         assert s1.concat == 's1sub|s1sub'
 
+    @testing.resolve_artifact_names
     def test_column_expression_joined(self):
         class Base(_base.ComparableEntity):
             pass
@@ -1206,6 +1216,7 @@ class OptimizedLoadTest(_base.MappedTest):
             ]
         )
 
+    @testing.resolve_artifact_names
     def test_composite_column_joined(self):
         class Base(_base.ComparableEntity):
             pass
@@ -1234,17 +1245,161 @@ class OptimizedLoadTest(_base.MappedTest):
         assert s2test.comp
         eq_(s1test.comp, Comp('ham', 'cheese'))
         eq_(s2test.comp, Comp('bacon', 'eggs'))
-        
-        
+
+    @testing.resolve_artifact_names
+    def test_load_expired_on_pending(self):
+        class Base(_base.ComparableEntity):
+            pass
+        class Sub(Base):
+            pass
+        mapper(Base, base, polymorphic_on=base.c.type, polymorphic_identity='base')
+        mapper(Sub, sub, inherits=Base, polymorphic_identity='sub')
+        sess = Session()
+        s1 = Sub(data='s1')
+        sess.add(s1)
+        self.assert_sql_execution(
+                testing.db,
+                sess.flush,
+                CompiledSQL(
+                    "INSERT INTO base (data, type) VALUES (:data, :type)",
+                    [{'data':'s1','type':'sub'}]
+                ),
+                CompiledSQL(
+                    "SELECT base.counter AS base_counter, "
+                    "sub.counter AS sub_counter FROM base JOIN sub ON base.id = "
+                    "sub.id WHERE base.id = :param_1",
+                    lambda ctx:{'param_1':s1.id}
+                ),
+                CompiledSQL(
+                    "INSERT INTO sub (id, sub) VALUES (:id, :sub)",
+                    lambda ctx:{'id':s1.id, 'sub':None}
+                ),
+        )
+
+    @testing.resolve_artifact_names
+    def test_dont_generate_on_none(self):
+        class Base(_base.ComparableEntity):
+            pass
+        class Sub(Base):
+            pass
+        mapper(Base, base, polymorphic_on=base.c.type, 
+                            polymorphic_identity='base')
+        m = mapper(Sub, sub, inherits=Base, polymorphic_identity='sub')
+
+        s1 = Sub()
+        assert m._optimized_get_statement(attributes.instance_state(s1), 
+                                ['counter2']) is None
+
+        # loads s1.id as None
+        eq_(s1.id, None)
+
+        # this now will come up with a value of None for id - should reject
+        assert m._optimized_get_statement(attributes.instance_state(s1), 
+                                ['counter2']) is None
+
+        s1.id = 1
+        attributes.instance_state(s1).commit_all(s1.__dict__, None)
+        assert m._optimized_get_statement(attributes.instance_state(s1), 
+                                ['counter2']) is not None
+
+    @testing.resolve_artifact_names
+    def test_load_expired_on_pending_twolevel(self):
+        class Base(_base.ComparableEntity):
+            pass
+        class Sub(Base):
+            pass
+        class SubSub(Sub):
+            pass
+
+        mapper(Base, base, polymorphic_on=base.c.type, 
+                    polymorphic_identity='base')
+        mapper(Sub, sub, inherits=Base, polymorphic_identity='sub')
+        mapper(SubSub, subsub, inherits=Sub, polymorphic_identity='subsub')
+        sess = Session()
+        s1 = SubSub(data='s1', counter=1)
+        sess.add(s1)
+        self.assert_sql_execution(
+                testing.db,
+                sess.flush,
+                CompiledSQL(
+                    "INSERT INTO base (data, type, counter) VALUES "
+                    "(:data, :type, :counter)",
+                    [{'data':'s1','type':'subsub','counter':1}]
+                ),
+                CompiledSQL(
+                    "INSERT INTO sub (id, sub, counter) VALUES "
+                    "(:id, :sub, :counter)",
+                    lambda ctx:[{'counter': 1, 'sub': None, 'id': s1.id}]
+                ),
+                CompiledSQL(
+                    "SELECT sub.counter2 AS sub_counter2, "
+                    "subsub.counter2 AS subsub_counter2 FROM base "
+                    "JOIN sub ON base.id = sub.id JOIN "
+                    "subsub ON sub.id = subsub.id WHERE base.id = :param_1",
+                    lambda ctx:{u'param_1': s1.id}
+                ),
+                CompiledSQL(
+                    "INSERT INTO subsub (id) VALUES (:id)",
+                    lambda ctx:{'id':s1.id}
+                ),
+        )
+
+
+
+class NoPKOnSubTableWarningTest(testing.TestBase):
+
+    def _fixture(self):
+        metadata = MetaData()
+        parent = Table('parent', metadata,
+            Column('id', Integer, primary_key=True)
+        )
+        child = Table('child', metadata,
+            Column('id', Integer, ForeignKey('parent.id'))
+        )
+        return parent, child
+
+    def tearDown(self):
+        clear_mappers()
+
+    def test_warning_on_sub(self):
+        parent, child = self._fixture()
+
+        class P(object):
+            pass
+        class C(P):
+            pass
+
+        mapper(P, parent)
+        assert_raises_message(
+            sa_exc.SAWarning,
+            "Could not assemble any primary keys for locally mapped "
+            "table 'child' - no rows will be persisted in this Table.",
+            mapper, C, child, inherits=P
+        )
+
+    def test_no_warning_with_explicit(self):
+        parent, child = self._fixture()
+
+        class P(object):
+            pass
+        class C(P):
+            pass
+
+        mapper(P, parent)
+        mc = mapper(C, child, inherits=P, primary_key=[parent.c.id])
+        eq_(mc.primary_key, (parent.c.id,))
+
 class PKDiscriminatorTest(_base.MappedTest):
     @classmethod
     def define_tables(cls, metadata):
         parents = Table('parents', metadata,
-                           Column('id', Integer, primary_key=True, test_needs_autoincrement=True),
+                           Column('id', Integer, primary_key=True, 
+                                    test_needs_autoincrement=True),
                            Column('name', String(60)))
-                           
+
         children = Table('children', metadata,
-                        Column('id', Integer, ForeignKey('parents.id'), primary_key=True),
+                        Column('id', Integer, ForeignKey('parents.id'), 
+                                    primary_key=True),
                         Column('type', Integer,primary_key=True),
                         Column('name', String(60)))
 
@@ -1260,13 +1415,13 @@ class PKDiscriminatorTest(_base.MappedTest):
 
         class A(Child):
             pass
-            
+
         mapper(Parent, parents, properties={
             'children': relationship(Child, backref='parent'),
         })
         mapper(Child, children, polymorphic_on=children.c.type,
             polymorphic_identity=1)
-            
+
         mapper(A, inherits=Child, polymorphic_identity=2)
 
         s = create_session()
@@ -1278,16 +1433,76 @@ class PKDiscriminatorTest(_base.MappedTest):
 
         assert a.id
         assert a.type == 2
-        
+
         p.name='p1new'
         a.name='a1new'
         s.flush()
-        
+
         s.expire_all()
         assert a.name=='a1new'
         assert p.name=='p1new'
-        
-        
+
+class NoPolyIdentInMiddleTest(_base.MappedTest):
+    @classmethod
+    def define_tables(cls, metadata):
+        Table('base', metadata,
+            Column('id', Integer, primary_key=True, 
+                            test_needs_autoincrement=True),
+            Column('type', String(50), nullable=False),
+        )
+
+    @classmethod
+    def setup_classes(cls):
+        class A(_base.ComparableEntity):
+            pass
+        class B(A):
+            pass
+        class C(B):
+            pass
+        class D(B):
+            pass
+        class E(A):
+            pass
+
+    @classmethod
+    @testing.resolve_artifact_names
+    def setup_mappers(cls):
+        mapper(A, base, polymorphic_on=base.c.type)
+        mapper(B, inherits=A, )
+        mapper(C, inherits=B, polymorphic_identity='c')
+        mapper(D, inherits=B, polymorphic_identity='d')
+        mapper(E, inherits=A, polymorphic_identity='e')
+
+    @testing.resolve_artifact_names
+    def test_load_from_middle(self):
+        s = Session()
+        s.add(C())
+        o = s.query(B).first()
+        eq_(o.type, 'c')
+        assert isinstance(o, C)
+
+    @testing.resolve_artifact_names
+    def test_load_from_base(self):
+        s = Session()
+        s.add(C())
+        o = s.query(A).first()
+        eq_(o.type, 'c')
+        assert isinstance(o, C)
+
+    @testing.resolve_artifact_names
+    def test_discriminator(self):
+        assert class_mapper(B).polymorphic_on is base.c.type
+        assert class_mapper(C).polymorphic_on is base.c.type
+
+    @testing.resolve_artifact_names
+    def test_load_multiple_from_middle(self):
+        s = Session()
+        s.add_all([C(), D(), E()])
+        eq_(
+            s.query(B).order_by(base.c.type).all(),
+            [C(), D()]
+        )
+
 class DeleteOrphanTest(_base.MappedTest):
     @classmethod
     def define_tables(cls, metadata):
@@ -1298,33 +1513,33 @@ class DeleteOrphanTest(_base.MappedTest):
             Column('data', String(50)),
             Column('parent_id', Integer, ForeignKey('parent.id'), nullable=False),
             )
-            
+
         parent = Table('parent', metadata,
                 Column('id', Integer, primary_key=True, test_needs_autoincrement=True),
                 Column('data', String(50))
             )
-    
+
     def test_orphan_message(self):
         class Base(_fixtures.Base):
             pass
-        
+
         class SubClass(Base):
             pass
-        
+
         class Parent(_fixtures.Base):
             pass
-        
+
         mapper(Base, single, polymorphic_on=single.c.type, polymorphic_identity='base')
         mapper(SubClass, inherits=Base, polymorphic_identity='sub')
         mapper(Parent, parent, properties={
             'related':relationship(Base, cascade="all, delete-orphan")
         })
-        
+
         sess = create_session()
         s1 = SubClass(data='s1')
         sess.add(s1)
         assert_raises_message(orm_exc.FlushError, 
             r"is not attached to any parent 'Parent' instance via "
             "that classes' 'related' attribute", sess.flush)
-        
-    
+
+
