@@ -13,7 +13,8 @@ from sqlalchemy.test.testing import assert_raises, assert_raises_message
 from sqlalchemy import Integer, String, ForeignKey, Sequence, exc as sa_exc
 from sqlalchemy.test.schema import Table
 from sqlalchemy.test.schema import Column
-from sqlalchemy.orm import mapper, relationship, create_session, class_mapper, backref, sessionmaker
+from sqlalchemy.orm import mapper, relationship, create_session, \
+    class_mapper, backref, sessionmaker, Session
 from sqlalchemy.orm import attributes, exc as orm_exc
 from sqlalchemy.test import testing
 from sqlalchemy.test.testing import eq_
@@ -40,15 +41,15 @@ class O2MCollectionTest(_fixtures.FixtureTest):
         u2= User(name='ed')
         sess.add_all([u1, a1, a2, a3])
         sess.commit()
-        
+
         #u1.addresses
-        
+
         def go():
             u2.addresses.append(a1)
             u2.addresses.append(a2)
             u2.addresses.append(a3)
         self.assert_sql_count(testing.db, go, 0)
-        
+
     @testing.resolve_artifact_names
     def test_collection_move_preloaded(self):
         sess = sessionmaker()()
@@ -87,7 +88,7 @@ class O2MCollectionTest(_fixtures.FixtureTest):
 
         # backref fires
         assert a1.user is u2
-        
+
         # u1.addresses wasn't loaded,
         # so when it loads its correct
         assert a1 not in u1.addresses
@@ -110,7 +111,7 @@ class O2MCollectionTest(_fixtures.FixtureTest):
 
         # backref fires
         assert a1.user is u2
-        
+
         # everything expires, no changes in 
         # u1.addresses, so all is fine
         sess.commit()
@@ -144,7 +145,7 @@ class O2MCollectionTest(_fixtures.FixtureTest):
     @testing.resolve_artifact_names
     def test_plain_load_passive(self):
         """test that many-to-one set doesn't load the old value."""
-        
+
         sess = sessionmaker()()
         u1 = User(name='jack')
         u2 = User(name='ed')
@@ -159,10 +160,10 @@ class O2MCollectionTest(_fixtures.FixtureTest):
         def go():
             a1.user = u2
         self.assert_sql_count(testing.db, go, 0)
-        
+
         assert a1 not in u1.addresses
         assert a1 in u2.addresses
-        
+
     @testing.resolve_artifact_names
     def test_set_none(self):
         sess = sessionmaker()()
@@ -176,11 +177,11 @@ class O2MCollectionTest(_fixtures.FixtureTest):
         def go():
             a1.user = None
         self.assert_sql_count(testing.db, go, 0)
-        
+
         assert a1 not in u1.addresses
-        
-        
-        
+
+
+
     @testing.resolve_artifact_names
     def test_scalar_move_notloaded(self):
         sess = sessionmaker()()
@@ -218,11 +219,12 @@ class O2MCollectionTest(_fixtures.FixtureTest):
         # "old" u1 here allows the backref
         # to remove it from the addresses collection
         a1.user = u2
-        
+
         sess.commit()
         assert a1 not in u1.addresses
         assert a1 in u2.addresses
 
+
 class O2OScalarBackrefMoveTest(_fixtures.FixtureTest):
     run_inserts = None
 
@@ -271,13 +273,13 @@ class O2OScalarBackrefMoveTest(_fixtures.FixtureTest):
 
         # load a1.user
         a1.user
-        
+
         # reassign
         a2.user = u1
 
         # backref fires
         assert u1.address is a2
-        
+
         # stays on both sides
         assert a1.user is u1
         assert a2.user is u1
@@ -298,7 +300,7 @@ class O2OScalarBackrefMoveTest(_fixtures.FixtureTest):
 
         # backref fires
         assert a1.user is u2
-        
+
         # u1.address loads now after a flush
         assert u1.address is None
         assert u2.address is a1
@@ -361,7 +363,7 @@ class O2OScalarBackrefMoveTest(_fixtures.FixtureTest):
 
         # load
         assert a1.user is u1
-        
+
         # reassign
         a2.user = u1
 
@@ -370,7 +372,7 @@ class O2OScalarBackrefMoveTest(_fixtures.FixtureTest):
 
         # didnt work this way tho
         assert a1.user is u1
-        
+
         # moves appropriately after commit
         sess.commit()
         assert u1.address is a2
@@ -428,19 +430,131 @@ class O2OScalarOrphanTest(_fixtures.FixtureTest):
         sess = sessionmaker()()
         a1 = Address(email_address="address1")
         u1 = User(name='jack', address=a1)
-        
+
         sess.add(u1)
         sess.commit()
         sess.expunge(u1)
-        
+
         u2= User(name='ed')
         # the _SingleParent extension sets the backref get to "active" !
         # u1 gets loaded and deleted
         u2.address = a1
         sess.commit()
         assert sess.query(User).count() == 1
-        
-    
+
+
+class M2MCollectionMoveTest(_fixtures.FixtureTest):
+    run_inserts = None
+
+    @classmethod
+    def setup_mappers(cls):
+        keywords, items, item_keywords, Keyword, Item = (cls.tables.keywords,
+                                cls.tables.items,
+                                cls.tables.item_keywords,
+                                cls.classes.Keyword,
+                                cls.classes.Item)
+
+        mapper(Item, items, properties={
+            'keywords':relationship(Keyword, secondary=item_keywords, 
+                                    backref='items')
+        })
+        mapper(Keyword, keywords)
+
+    def test_add_remove_pending_backref(self):
+        """test that pending doesn't add an item that's not a net add."""
+
+        Item, Keyword = (self.classes.Item, self.classes.Keyword)
+
+        session = Session(autoflush=False)
+
+        i1 = Item(description='i1')
+        session.add(i1)
+        session.commit()
+
+        session.expire(i1, ['keywords'])
+
+        k1= Keyword(name='k1')
+        k1.items.append(i1)
+        k1.items.remove(i1)
+        eq_(i1.keywords, [])
+
+    def test_remove_add_pending_backref(self):
+        """test that pending doesn't remove an item that's not a net remove."""
+
+        Item, Keyword = (self.classes.Item, self.classes.Keyword)
+
+        session = Session(autoflush=False)
+
+        k1= Keyword(name='k1')
+        i1 = Item(description='i1', keywords=[k1])
+        session.add(i1)
+        session.commit()
+
+        session.expire(i1, ['keywords'])
+
+        k1.items.remove(i1)
+        k1.items.append(i1)
+        eq_(i1.keywords, [k1])
+
+    def test_pending_combines_with_flushed(self):
+        """test the combination of unflushed pending + lazy loaded from DB."""
+
+        Item, Keyword = (self.classes.Item, self.classes.Keyword)
+
+        session = Session(testing.db, autoflush=False)
+
+        k1 = Keyword(name='k1')
+        k2 = Keyword(name='k2')
+        i1 = Item(description='i1', keywords=[k1])
+        session.add(i1)
+        session.add(k2)
+        session.commit()
+
+        k2.items.append(i1)
+        # the pending
+        # list is still here.
+        eq_(
+            set(attributes.instance_state(i1).
+                pending['keywords'].added_items),
+            set([k2])
+        )
+        # because autoflush is off, k2 is still
+        # coming in from pending
+        eq_(i1.keywords, [k1, k2])
+
+        # prove it didn't flush
+        eq_(session.scalar("select count(*) from item_keywords"), 1)
+
+        # the pending collection was removed
+        assert 'keywords' not in attributes.\
+                                instance_state(i1).\
+                                pending
+
+    def test_duplicate_adds(self):
+        Item, Keyword = (self.classes.Item, self.classes.Keyword)
+
+        session = Session(testing.db, autoflush=False)
+
+        k1 = Keyword(name='k1')
+        i1 = Item(description='i1', keywords=[k1])
+        session.add(i1)
+        session.commit()
+
+        k1.items.append(i1)
+        eq_(i1.keywords, [k1, k1])
+
+        session.expire(i1, ['keywords'])
+        k1.items.append(i1)
+        eq_(i1.keywords, [k1, k1])
+
+        session.expire(i1, ['keywords'])
+        k1.items.append(i1)
+        eq_(i1.keywords, [k1, k1])
+
+        eq_(k1.items, [i1, i1, i1, i1])
+        session.commit()
+        eq_(k1.items, [i1])
+
 class M2MScalarMoveTest(_fixtures.FixtureTest):
     run_inserts = None
 
@@ -448,28 +562,30 @@ class M2MScalarMoveTest(_fixtures.FixtureTest):
     @testing.resolve_artifact_names
     def setup_mappers(cls):
         mapper(Item, items, properties={
-            'keyword':relationship(Keyword, secondary=item_keywords, uselist=False, backref=backref("item", uselist=False))
+            'keyword':relationship(Keyword, secondary=item_keywords, 
+                                    uselist=False, 
+                                    backref=backref("item", uselist=False))
         })
         mapper(Keyword, keywords)
-    
+
     @testing.resolve_artifact_names
     def test_collection_move_preloaded(self):
         sess = sessionmaker()()
-        
+
         k1 = Keyword(name='k1')
         i1 = Item(description='i1', keyword=k1)
         i2 = Item(description='i2')
 
         sess.add_all([i1, i2, k1])
         sess.commit() # everything is expired
-        
+
         # load i1.keyword
         assert i1.keyword is k1
-        
+
         i2.keyword = k1
 
         assert k1.item is i2
-        
+
         # nothing happens.
         assert i1.keyword is k1
         assert i2.keyword is k1
